#!/bin/sh

# This tool copies files, directories, special files and directory trees
# exactly as they are, and tries to preserve the file owner and permission
# bits as best as possible.

# However, the owner of soft link copies is set to the user performing the
# copy, their file modification date and time is set to the current date
# and time, and the group is set to either the group of the target directory
# (if the user performing the copy is not root) or the group of the source
# directory (if the user performing the copy is root), provided the soft
# links are inside a directory. If they appear on the command line, the
# group of their copies is always set to the group of the target directory.

# At least according to my extensive tests under FreeBSD 2.2.6-RELEASE. :-)

# (This mess is not my fault - "tar" and "ln -s" really don't seem to know
# what they're doing at times! ;-) )

# Moreover, if the user performing the copy is not root, the group of ALL
# copies will be set to the group of the target directory.

# (To prevent this, simply create a temporary directory with the desired
# ownership which will serve as the target directory. After the copy,
# move its contents up into its parent directory and delete it.)

# This tool has one major advantage over copying with "cp -RPp":
# It preserves hard links and handles hard links inside and
# outside of the copied directory tree correctly.

# Copyright (c) 2000 by Steffen Beyer. All rights reserved.

# This tool can be used, modified and redistributed under the terms of
# (at your option) either the FreeBSD license, the GNU General Public
# License (version 2.0 or higher) or Perl's Artistic License.

self=`basename $0 .sh`

if [ $# -lt 2 ]
then
    echo "Usage: $self (DIR|FILE)+ TARGETDIR"
    exit 1
fi

args=''
while [ $# -gt 1 ]
do
    item="$1"
    if [ ! -e "$item" ] && [ ! -h "$item" ]
    then
        echo "$self: item '$item': no such file or directory!" >&2
        exit 1
    fi
    if [ "x$args" = "x" ]
    then
        args="'$item'"
    else
        args="$args '$item'"
    fi
    shift
done

target="$1"
shift

if [ "x$target" = "x." ]
then
    echo "$self: target '$target' is not allowed!" >&2
    exit 1
fi
if [ ! -d "$target" ]
then
    echo "$self: target directory '$target' does not exist!" >&2
    exit 1
fi

echo "/usr/bin/tar cf - $args | ( cd '$target'; tar xBpf - )"
eval "/usr/bin/tar cf - $args | ( cd '$target'; tar xBpf - )"

